#!/usr/local/bin/perl
# qt2www.pl <quark tag map> <quark tag source> <html output>
#   <quark tag map> 	a file describing how tags should be converted to HTML
#			see README for details
#   <quark tag source>  a text file marked with Quark Tags
#                       see your Quark manual for an explanation of how to
#			export text with Quark Tags 
#   <html output>       file which HTML output will be written to

# qt2www converts plain text marked with QuarkXPress Tags to HTML

# Copyright 1994-5, The Tech
# By Jeremy Hylton
# jeremy@the-tech.mit.edu
# based on q2w3.pl by Jeremy Hylton and Reuven Lerner
# ------------------------------------------------------------
# qt2www is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.

# qt2www is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
# License for more details.

# You should have received a copy of the GNU General Public License
# along with qt2www; see the file COPYING.  If not, write to the Free
# Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
# ------------------------------------------------------------

# Revision History
# July 11	Jer	Wrote functional first draft
# July 24	Jer	Added support for bold/italic
# Aug  18       Jer     Fixed b/i error (<=, not <)
# Aug  18       Jer     changed &status to pass status variable
# Aug  25       Jer     add n-dash, degree support
# Aug  31       Jer     added processing for tags that extend over \n
# Aug  31       Jer     fixed regexp to clear unprocessed tags
# Sept 21       Jer     now recognizes quark tag names with spaces
# Sept 21       Jer     has a default type used for unknown tags
# Sept 21       Jer     fixed bug that ignored the first character after
#		        quote marks when reading font info; nasty!
# Sept 22       Jer     Fixed comma separation in quark tags
# Sept 23       Jer     Fixed bug for end of B or I on new style tag
# Sept 23       Jer     Fixed bug which ignored half of <BI>
# Sept 27       Jer     Fixed translation of __w__ to <w>
# Sept 27       Jer     Turn off B and I at the end of the file
# Oct  20       Jer     Removed spurious change from true of false for B&I.
#                       Problem occurred when turning off BI from last tag.
# Nov  14       Jer     Began support for HTML 2.0
# Dec   6       Jer     revamped to include %STATE and tag subroutines
# Dec   6       Jer     fixed some bold/italic problems, reading the map file
# Dec   6       Jer     added support for the emphasis field of the map
# Feb  27 1995  Jer     added a usage routine

require "qt2www-inc.pl";

&usage if ($#ARGV != 2);
($map, $source, $output) = @ARGV;

$status_output = 'yes';
# change this variable to disable to status updates during run

&status ($status_output, "qt2www $version\n");

&status ($status_output, "Reading QuarkTag to HTML mapping.\n");
&read_map ($map);

open (SRC, "$source") || die "Could not open source file ($source).\n";
open (OUT, ">$output") || die "Could not create output file ($output).\n";

&status ($status_output, "Processing QuarkTag source file.\n");

# begin with the proper HTML 2.0 headers
print OUT "<!doctype HTML public \"-//W30//DTD W3 HTML 2.0//EN\">\n";
print OUT "<HTML>\n";
print OUT "<HEAD><TITLE>\n</TITLE></HEAD>\n";
# don't know what to do for the HEAD tag
print OUT "<BODY>\n";

close (SRC);
close (OUT);


&MainProcessingRoutine ($source, $output);


open (SRC, "$source") || die "Could not open source file ($source).\n";
open (OUT, ">>$output") || die "Could not open output file ($output).\n";

# close up the HTML tag opened by the header    
print OUT "</BODY>\n";
print OUT "</HTML>\n";

close (OUT);
close (SRC);

&status ($status_output, "Finished.\n");


exit 0;

sub usage
{
    print <<"EOM";
qt2www converts plain text marked with QuarkXPress Tags to HTML
Version $version

qt2www.pl <quark tag map> <quark tag source> <html output>
   <quark tag map>     a file describing how tags should be converted to HTML
		       see README for details
   <quark tag source>  a text file marked with XPress Tags
                       see your Quark manual for an explanation of how to
		       export text with XPress Tags 
   <html output>       file which HTML output will be written to
EOM

    exit 0;
}


